import os
import subprocess
import shutil
from django.conf import settings
from django.core.management.base import BaseCommand
from django.utils import timezone
from django_commands_suite.utils import log_command

class Command(BaseCommand):
   """
      Backup the database into a `.sql` file.

      Supported databases:
      - SQLite
      - PostgreSQL
      - MySQL

      Examples:
         python manage.py backup_db --db=sqlite
   """
   help = "Backup your database (SQLite, PostgreSQL, or MySQL)."
   
   def add_arguments(self, parser):
         parser.add_argument(
            "--db",
            type=str,
            choices=["sqlite", "postgres", "mysql"],
            required=True,
            help="Database type: sqlite, postgres, or mysql",
         )

   def handle(self, *args, **options):
      db_type = options["db"]
      timestamp = timezone.now().strftime("%Y%m%d%H%M%S")
      output_file = f"backup_db{timestamp}.sql"

      db = settings.DATABASES["default"]
      name = db["NAME"]
      user = db.get("USER", "")
      password = db.get("PASSWORD", "")
      host = db.get("HOST", "localhost")
      port = db.get("PORT", "")

      try:
         if db_type == "sqlite":
               output_file = f"backup_db{timestamp}.sqlite3"
               shutil.copy(name, output_file)

         elif db_type == "postgres":
            env = os.environ.copy()
            env["PGPASSWORD"] = password
            cmd = [
               "pg_dump",
               "-h", host,
               "-p", str(port or 5432),
               "-U", user,
               "-f", output_file,
               name,
            ]
            subprocess.run(cmd, env=env, check=True)

         elif db_type == "mysql":
            cmd = [
               "mysqldump",
               "-h", host,
               "-P", str(port or 3306),
               "-u", user,
               f"-p{password}",
               name,
            ]
            with open(output_file, "w") as f:
               subprocess.run(cmd, stdout=f, check=True)

         log_command("backup_db" , args , "success" ,f"backup saved in {output_file}")
         self.stdout.write(self.style.SUCCESS(f"Backup saved to {output_file}"))

      except Exception as e:
            log_command("backup_db" , args , "error" ,f"Backup failed: {e}")
            self.stdout.write(self.style.ERROR(f"Backup failed: {e}"))



import shutil, os
from django.core.management.base import BaseCommand
from django.conf import settings
from django.utils.timezone import now
from django_commands_suite.utils import log_command

class Command(BaseCommand):
   
   help = "Backup MEDIA_ROOT into a zip file."

   def handle(self, *args, **options):
      try:
         output_file = f"media_backup_{now().strftime('%Y%m%d%H%M%S')}.zip"
         shutil.make_archive(output_file.replace(".zip",""), 'zip', settings.MEDIA_ROOT)
         
         self.stdout.write(self.style.SUCCESS(f"Media backup created: {output_file}"))
         log_command("backup_media", args, "success", f"File: {output_file}")
      
      except Exception as e:
         self.stderr.write(str(e))
         log_command("backup_media", args, "error", str(e))



from django.db import models
from django.utils import timezone

class CommandLog(models.Model):
   name = models.CharField(max_length=100)   
   args = models.TextField(blank=True, null=True)  
   executed_at = models.DateTimeField(default=timezone.now)  
   status = models.CharField(max_length=20, choices=[
      ("success", "Success"),
      ("error", "Error"),
   ])
   message = models.TextField(blank=True, null=True)  

   def __str__(self):
      return f"{self.name} - {self.status} @ {self.executed_at}"



from django.core.management.base import BaseCommand
from django.core.management import get_commands, load_command_class
import inspect
import textwrap

class Command(BaseCommand):
    help = "Show all available commands in Django Commands Suite (DCS) with usage examples"

    def handle(self, *args, **kwargs):
        self.stdout.write(self.style.SUCCESS("Django Commands Suite (DCS)"))
        self.stdout.write("Available commands:\n")

        commands = get_commands()

        for name, app in commands.items():
            if app == "django_commands_suite":
                try:
                    cmd_class = load_command_class(app, name)
                    usage = getattr(cmd_class, "help", "No description provided.")
                    doc = inspect.getdoc(cmd_class) or ""
                except Exception:
                    usage = "No description available."
                    doc = ""

                self.stdout.write("=" * 60)
                self.stdout.write(f"    {name}")
                self.stdout.write(f"    Description: {usage}")
                self.stdout.write(f"    Usage: python manage.py {name}")

                # extract examples from docstring if available
                if "Examples:" in doc:
                    self.stdout.write("")
                    self.stdout.write("    Examples:")
                    for line in doc.splitlines():
                        if line.strip().startswith("python manage.py"):
                            self.stdout.write("     " + textwrap.dedent(line))
                self.stdout.write("")
        self.stdout.write("=" * 60)



from django.contrib import admin 
from .models import CommandLog



@admin.register(CommandLog)
class CommandLog(admin.ModelAdmin) : 
   list_display  = [ "id","name" , "status" ,"executed_at"]
   search_fields = ['name']
   list_filter = ["status"]




from django.apps import AppConfig

class DjangoCommandsSuiteConfig(AppConfig):
   default_auto_field = 'django.db.models.BigAutoField'
   name = 'django_commands_suite'
   verbose_name = "Django Commands Suite"



from django.core.management.base import BaseCommand
from django.core.management.utils import get_random_secret_key
from django_commands_suite.utils import log_command

class Command(BaseCommand):
   
   help = "Generate a new Django SECRET_KEY."

   def handle(self, *args, **options):
      try:
         key = get_random_secret_key()
         self.stdout.write(self.style.SUCCESS(f"New SECRET_KEY: {key}"))
         log_command("generate_secret_key", args, "success", key)
      except Exception as e:
         self.stderr.write(str(e))
         log_command("generate_secret_key", args, "error", str(e))



from django.core.management import call_command
from .models import CommandLog
import io

def log_command(name, args=None, status="success", message=""):
   CommandLog.objects.create(
      name=name,
      args=str(args),
      status=status,
      message=message
   )





def run_command(command_name: str, *args, **kwargs):
    """
    Run a Django management command from code (instead of CLI).
    It will capture output and log it automatically into CommandLog.
    """
    out = io.StringIO()
    try:
        call_command(command_name, *args, stdout=out, stderr=out, **kwargs)
        output = out.getvalue()
        CommandLog.objects.create(
            command=command_name,
            status="success",
            message=output,
            options=str(kwargs)
        )
        return output
    except Exception as e:
        CommandLog.objects.create(
            command=command_name,
            status="error",
            message=str(e),
            options=str(kwargs)
        )
        raise e



from django.urls import path
from . import views

urlpatterns = [
   path("dcs_terminal/", views.terminal_page, name="dcs_terminal"),
   #path("dcs_terminal/run/", views.run_command, name="dcs_run_command"),
]



from django.core.management.base import BaseCommand
from django.contrib.auth import get_user_model
from django_commands_suite.utils import log_command


class Command(BaseCommand):
   help = "Quickly create a default superuser (username=djadmin, password=djadmin, email=admin@dj.com)"

   def handle(self, *args, **options):
      User = get_user_model()
      if not User.objects.filter(username="djadmin").exists():
            User.objects.create_superuser(
               username="djadmin",
               email="admin@dj.com",
               password="djadmin"
            )
            log_command("quick_superuser", args ,"success","Superuser 'djadmin' created successfully")
            self.stdout.write(self.style.SUCCESS("Superuser 'djadmin' created successfully!"))
      else:
            log_command("quick_superuser", args ,"error", "Superuser 'djadmin' already exists.")
            self.stdout.write(self.style.WARNING("Superuser 'djadmin' already exists."))



from django.contrib.sessions.models import Session
from django.core.management.base import BaseCommand
from django_commands_suite.utils import log_command

class Command(BaseCommand):
   
   help = "Flush all sessions from database ."

   def handle(self, *args, **options):
      try:
         count, _ = Session.objects.all().delete()
         self.stdout.write(self.style.SUCCESS(f"Deleted {count} sessions."))
         log_command("flush_sessions", args, "success", f"{count} deleted")
      except Exception as e:
         self.stderr.write(str(e))
         log_command("flush_sessions", args, "error", str(e))



from django.apps import apps
from django.core.management.base import BaseCommand
from faker import Faker
from django.db import models
from django_commands_suite.utils import log_command

fake = Faker()

class Command(BaseCommand):
    """
    Insert dummy data into a model with realistic fake values.

    Examples:
        python manage.py dummy_data --model app_label.ModelName --count 10
    """

    help = "Insert dummy data into a model with realistic fake values using mapping instead of if-else."

    def add_arguments(self, parser):
        parser.add_argument("--model", type=str, required=True, help="app.ModelName")
        parser.add_argument("--count", type=int, default=2)

    def handle(self, *args, **options):
        try:
            app_label, model_name = options["model"].split(".")
            model = apps.get_model(app_label, model_name)

            generators = {
                models.CharField: lambda f: fake.text(getattr(f, "max_length", 20))[: getattr(f, "max_length", 20)],
                models.TextField: lambda f: fake.paragraph(),
                models.EmailField: lambda f: fake.email(),
                models.URLField: lambda f: fake.url(),
                models.IntegerField: lambda f: fake.random_int(),
                models.FloatField: lambda f: fake.pyfloat(left_digits=3, right_digits=2),
                models.DateField: lambda f: fake.date(),
                models.DateTimeField: lambda f: fake.date_time(),
                models.BooleanField: lambda f: fake.boolean(),
                models.DecimalField: lambda f: fake.pydecimal(
                    left_digits=f.max_digits - f.decimal_places,
                    right_digits=f.decimal_places,
                ),
            }

            fields = [f for f in model._meta.get_fields() if getattr(f, "editable", False) and not f.auto_created]

            created_count = 0
            for _ in range(options["count"]):
                data = {}
                for field in fields:
                    generator = next(
                        (gen for field_type, gen in generators.items() if isinstance(field, field_type)),
                        None,
                    )
                    if generator:
                        data[field.name] = generator(field)

                obj = model.objects.create(**data)
                created_count += 1

            self.stdout.write(self.style.SUCCESS(f"{created_count} dummy objects created for {options['model']}"))
            log_command("create_dummy_data", options, "success", f"{created_count} objects created")

        except Exception as e:
            self.stderr.write(self.style.ERROR(str(e)))
            log_command("create_dummy_data", options, "error", str(e))
